package Test::Repocop::FixscriptFactory;

use strict;
use warnings;
use Carp;
use Test::Repocop::Common;
use Test::Repocop::RepocopResource;
use RPM::Source::Transformation::OptHandlerWrapper;
use RPM::Source::Transformation::Factory;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;
our $VERSION=0.003;

our $verbose=1;

sub register_messages {
    my (undef,$extopt)=@_;
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-repocop_sourceid => undef,
	-repocop_testnames => [],
	-repocop_pkgnames => [],
    })
}

sub transformations {
    my (undef,$opt,$resourceobj)=@_;
    my @transformations = (
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_SRPM_POST_BEGIN,
	    -name => 'initialize default changelog',
	    -opt_handler_sub=>\&opt_handler_add_default_changelog_AT_BEGIN)
	);

    my $testnames=$opt->{-repocop_testnames};
    for (my $i=0;$i<@$testnames;$i++) {
	push @transformations, RPM::Source::Transformation::RepocopFixWrapper->new(
	    -testname => $testnames->[$i],
	    -pkgname => $opt->{-repocop_pkgnames}->[$i],
	    -drop_me_test_modification => undef,
	    );
    }
    # postclean scripts
    foreach my $name (@Test::Repocop::RepocopResource::postcleanscripts) {
	push @transformations, RPM::Source::Transformation::RepocopFixWrapper->new(
	    -testname => $name,
	    -pkgname => '',
	    -drop_me_test_modification => 1,
	    );
    }
    return @transformations;
}

sub opt_handler_add_default_changelog_AT_BEGIN {
    my (undef,$spec,$parent,$opt,$resourceobj)=@_;
    $spec->add_default_changelog('- applied repocop fixes:'."\n");
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

package RPM::Source::Transformation::RepocopFixWrapper;

use strict;
use warnings;

our $verbose=0;

sub new {
    my $class = shift;
    my $self = {
	-priority => 0,
	@_
    };
    die "new: -testname is required" unless defined $self->{-testname};
    die "new: -pkgname is required" unless defined $self->{-pkgname};
    
    $self->{-subject}='repocop fix for '.$self->{-testname};
    $self->{-name}=$self->{-testname};
    # TODO !!! load description! # using lazy load cached sub in RepocopResource
    #$self->{-description}=$description{$self->{-testname}};
    bless $self, $class;
    return $self;
}

sub apply {
    my ($self,$spec,$parent,$opt,$resourceobj)=@_;
    die "$spec is undefined" unless $spec;
    &_call_fixscript($spec,$parent,$self->{-testname},$self->{-pkgname},$self->{-drop_me_test_modification});
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub _call_fixscript {
    my ($spec,$parent,$testname,$pkg, $test_modification)=@_;
    my $msg_pkg=$pkg;
    $msg_pkg||='the spec file';
    print STDERR "trying $testname for $msg_pkg\n" if $verbose>1;
    my $fixscriptref=&__load_pl($testname);
    if ($fixscriptref) {
	print STDERR "applying $testname for $msg_pkg\n" if $verbose;
	&Test::Repocop::Common::set_test_environment($testname);
	my $before=$spec->get_spec() if $test_modification;
#	$spec->{WARN_NOTAPPLIED}++;
	&{$fixscriptref}($spec,$pkg,$pkg);
#	$spec->{WARN_NOTAPPLIED}--;

	return if $test_modification and $spec->get_spec() eq $before;
	$spec->add_default_changelog("  * $testname for $msg_pkg\n");
	print STDERR "applied $testname for $msg_pkg\n" if $verbose;
    } else {
	print STDERR "no fixscript for $testname for $msg_pkg\n" if $verbose;
    }
}

our @SPECHOOKS;
my %fixscriptmap;

sub __load_pl {
    my $testname=shift;
    print STDERR "Repocop::FixscriptFactory: __load_pl: proceed with $testname:\n" if $verbose>3;
    unless ($fixscriptmap{$testname}) {
	eval {
	    require $testname.'.pl';
	};
	warn "can't load fixscript for $testname: skipped\nmsg:$@\n" if $@ and $verbose;
	if (@SPECHOOKS) {
	    $fixscriptmap{$testname}=$SPECHOOKS[0];
	    $#{SPECHOOKS}=-1;
	} else {
	    print STDERR "ERROR: Repocop::FixscriptFactory: fixscript for $testname does not have a hook!\n";
	}
    } else {
	print STDERR "Repocop::FixscriptFactory: __load_pl: fixscriptmap exists:$fixscriptmap{$testname}\n" if $verbose>3;
    }
    return $fixscriptmap{$testname};
}

1;
